"""
Literature examples implemented in Abjad.
"""

from abjad.demos import bartok
from abjad.demos import ferneyhough
from abjad.demos import ligeti
from abjad.demos import mozart
from abjad.demos import part
