lilypond_version = "2.19.24"

interface_properties = {
    "accidental-interface": [
        "alteration",
        "avoid-slur",
        "glyph-name",
        "glyph-name-alist",
        "hide-tied-accidental-after-break",
        "parenthesized",
        "restore-first",
        ],
    "accidental-placement-interface": [
        "direction",
        "padding",
        "positioning-done",
        "right-padding",
        "script-priority",
        ],
    "accidental-suggestion-interface": [],
    "align-interface": [
        "align-dir",
        "axes",
        "padding",
        "positioning-done",
        "stacking-dir",
        ],
    "ambitus-interface": [
        "gap",
        "length-fraction",
        "maximum-gap",
        "thickness",
        ],
    "arpeggio-interface": [
        "arpeggio-direction",
        "dash-definition",
        "positions",
        "protrusion",
        "script-priority",
        ],
    "axis-group-interface": [
        "axes",
        "default-staff-staff-spacing",
        "max-stretch",
        "no-alignment",
        "nonstaff-nonstaff-spacing",
        "nonstaff-relatedstaff-spacing",
        "nonstaff-unrelatedstaff-spacing",
        "staff-affinity",
        "staff-staff-spacing",
        ],
    "balloon-interface": [
        "annotation-balloon",
        "annotation-line",
        "padding",
        "text",
        ],
    "bar-line-interface": [
        "allow-span-bar",
        "bar-extent",
        "gap",
        "glyph",
        "glyph-name",
        "hair-thickness",
        "kern",
        "rounded",
        "segno-kern",
        "thick-thickness",
        ],
    "bass-figure-alignment-interface": [],
    "bass-figure-interface": [
        "implicit",
        ],
    "beam-interface": [
        "X-positions",
        "annotation",
        "auto-knee-gap",
        "beam-thickness",
        "beamed-stem-shorten",
        "beaming",
        "break-overshoot",
        "clip-edges",
        "collision-interfaces",
        "collision-voice-only",
        "concaveness",
        "damping",
        "details",
        "direction",
        "gap",
        "gap-count",
        "grow-direction",
        "inspect-quants",
        "knee",
        "length-fraction",
        "neutral-direction",
        "positions",
        "skip-quanting",
        ],
    "bend-after-interface": [
        "thickness",
        ],
    "break-alignable-interface": [
        "break-align-symbols",
        "non-break-align-symbols",
        ],
    "break-aligned-interface": [
        "break-align-anchor",
        "break-align-anchor-alignment",
        "break-align-symbol",
        "space-alist",
        ],
    "break-alignment-interface": [
        "break-align-orders",
        "positioning-done",
        ],
    "breathing-sign-interface": [
        "direction",
        ],
    "chord-name-interface": [],
    "clef-interface": [
        "full-size-change",
        "glyph",
        "glyph-name",
        "non-default",
        ],
    "clef-modifier-interface": [
        "clef-alignments",
        ],
    "cluster-beacon-interface": [
        "positions",
        ],
    "cluster-interface": [
        "padding",
        "style",
        ],
    "custos-interface": [
        "neutral-direction",
        "neutral-position",
        "style",
        ],
    "dot-column-interface": [
        "chord-dots-limit",
        "direction",
        "positioning-done",
        ],
    "dots-interface": [
        "direction",
        "dot-count",
        "style",
        ],
    "dynamic-interface": [],
    "dynamic-line-spanner-interface": [
        "avoid-slur",
        ],
    "dynamic-text-interface": [
        "right-padding",
        ],
    "dynamic-text-spanner-interface": [
        "text",
        ],
    "enclosing-bracket-interface": [
        "bracket-flare",
        "edge-height",
        "padding",
        "shorten-pair",
        "thickness",
        ],
    "episema-interface": [],
    "figured-bass-continuation-interface": [
        "padding",
        "thickness",
        ],
    "finger-interface": [],
    "fingering-column-interface": [
        "padding",
        "positioning-done",
        "snap-radius",
        ],
    "flag-interface": [
        "glyph-name",
        "stroke-style",
        "style",
        ],
    "font-interface": [
        "font-encoding",
        "font-family",
        "font-name",
        "font-series",
        "font-shape",
        "font-size",
        ],
    "footnote-interface": [
        "automatically-numbered",
        "footnote",
        "footnote-text",
        ],
    "footnote-spanner-interface": [
        "footnote-text",
        ],
    "fret-diagram-interface": [
        "align-dir",
        "dot-placement-list",
        "fret-diagram-details",
        "size",
        "thickness",
        ],
    "glissando-interface": [],
    "grace-spacing-interface": [
        "common-shortest-duration",
        ],
    "gregorian-ligature-interface": [],
    "grid-line-interface": [
        "thickness",
        ],
    "grid-point-interface": [],
    "grob-interface": [
        "X-extent",
        "X-offset",
        "Y-extent",
        "Y-offset",
        "after-line-breaking",
        "avoid-slur",
        "before-line-breaking",
        "color",
        "cross-staff",
        "extra-offset",
        "footnote-music",
        "forced-spacing",
        "horizontal-skylines",
        "id",
        "layer",
        "minimum-X-extent",
        "minimum-Y-extent",
        "parenthesis-friends",
        "rotation",
        "skyline-horizontal-padding",
        "springs-and-rods",
        "stencil",
        "transparent",
        "vertical-skylines",
        "whiteout",
        "whiteout-box",
        ],
    "hairpin-interface": [
        "bound-padding",
        "broken-bound-padding",
        "circled-tip",
        "grow-direction",
        "height",
        ],
    "hara-kiri-group-spanner-interface": [
        "remove-empty",
        "remove-first",
        "remove-layer",
        ],
    "horizontal-bracket-interface": [
        "bracket-flare",
        "connect-to-neighbor",
        "edge-height",
        "shorten-pair",
        ],
    "inline-accidental-interface": [],
    "instrument-specific-markup-interface": [
        "fret-diagram-details",
        "graphical",
        "harp-pedal-details",
        "size",
        "thickness",
        ],
    "item-interface": [
        "break-visibility",
        "extra-spacing-height",
        "extra-spacing-width",
        "non-musical",
        ],
    "key-cancellation-interface": [],
    "key-signature-interface": [
        "alteration-alist",
        "flat-positions",
        "glyph-name-alist",
        "padding",
        "padding-pairs",
        "sharp-positions",
        ],
    "kievan-ligature-interface": [
        "padding",
        ],
    "ledger-line-spanner-interface": [
        "gap",
        "length-fraction",
        "minimum-length-fraction",
        ],
    "ledgered-interface": [
        "no-ledgers",
        ],
    "ligature-bracket-interface": [
        "height",
        "thickness",
        "width",
        ],
    "ligature-head-interface": [],
    "ligature-interface": [],
    "line-interface": [
        "arrow-length",
        "arrow-width",
        "dash-fraction",
        "dash-period",
        "style",
        "thickness",
        "zigzag-length",
        "zigzag-width",
        ],
    "line-spanner-interface": [
        "bound-details",
        "extra-dy",
        "gap",
        "left-bound-info",
        "right-bound-info",
        "simple-Y",
        "thickness",
        "to-barline",
        ],
    "lyric-extender-interface": [
        "left-padding",
        "next",
        "right-padding",
        "thickness",
        ],
    "lyric-hyphen-interface": [
        "dash-period",
        "height",
        "length",
        "minimum-distance",
        "minimum-length",
        "padding",
        "thickness",
        ],
    "lyric-interface": [],
    "lyric-syllable-interface": [],
    "mark-interface": [],
    "measure-counter-interface": [
        "count-from",
        ],
    "measure-grouping-interface": [
        "height",
        "style",
        "thickness",
        ],
    "melody-spanner-interface": [
        "neutral-direction",
        ],
    "mensural-ligature-interface": [
        "thickness",
        ],
    "metronome-mark-interface": [],
    "multi-measure-interface": [
        "bound-padding",
        ],
    "multi-measure-rest-interface": [
        "bound-padding",
        "expand-limit",
        "hair-thickness",
        "measure-count",
        "minimum-length",
        "round-up-exceptions",
        "round-up-to-longer-rest",
        "spacing-pair",
        "thick-thickness",
        "usable-duration-logs",
        ],
    "note-collision-interface": [
        "merge-differently-dotted",
        "merge-differently-headed",
        "positioning-done",
        "prefer-dotted-right",
        ],
    "note-column-interface": [
        "force-hshift",
        "horizontal-shift",
        "ignore-collision",
        ],
    "note-head-interface": [
        "duration-log",
        "glyph-name",
        "ignore-ambitus",
        "note-names",
        "stem-attachment",
        "style",
        ],
    "note-name-interface": [],
    "note-spacing-interface": [
        "knee-spacing-correction",
        "same-direction-correction",
        "space-to-barline",
        "stem-spacing-correction",
        ],
    "number-interface": [
        "number-type",
        ],
    "only-prebreak-interface": [],
    "ottava-bracket-interface": [
        "bracket-flare",
        "edge-height",
        "minimum-length",
        "shorten-pair",
        ],
    "outside-staff-axis-group-interface": [
        "outside-staff-placement-directive",
        ],
    "outside-staff-interface": [
        "outside-staff-horizontal-padding",
        "outside-staff-padding",
        "outside-staff-priority",
        ],
    "paper-column-interface": [
        "between-cols",
        "full-measure-extra-space",
        "labels",
        "line-break-penalty",
        "line-break-permission",
        "line-break-system-details",
        "page-break-penalty",
        "page-break-permission",
        "page-turn-penalty",
        "page-turn-permission",
        "rhythmic-location",
        "shortest-playing-duration",
        "shortest-starter-duration",
        "used",
        "when",
        ],
    "parentheses-interface": [
        "padding",
        "stencils",
        ],
    "percent-repeat-interface": [
        "dot-negative-kern",
        "slash-negative-kern",
        "slope",
        "thickness",
        ],
    "percent-repeat-item-interface": [
        "dot-negative-kern",
        "slash-negative-kern",
        "slope",
        "thickness",
        ],
    "piano-pedal-bracket-interface": [
        "bound-padding",
        "bracket-flare",
        "edge-height",
        "shorten-pair",
        ],
    "piano-pedal-interface": [],
    "piano-pedal-script-interface": [],
    "pitched-trill-interface": [],
    "pure-from-neighbor-interface": [],
    "rest-collision-interface": [
        "minimum-distance",
        "positioning-done",
        ],
    "rest-interface": [
        "direction",
        "minimum-distance",
        "style",
        "voiced-position",
        ],
    "rhythmic-grob-interface": [],
    "rhythmic-head-interface": [
        "duration-log",
        "glissando-skip",
        ],
    "script-column-interface": [],
    "script-interface": [
        "avoid-slur",
        "positioning-done",
        "script-priority",
        "side-relative-direction",
        "slur-padding",
        "toward-stem-shift",
        "toward-stem-shift-in-column",
        ],
    "self-alignment-interface": [
        "X-align-on-main-noteheads",
        "parent-alignment-X",
        "parent-alignment-Y",
        "self-alignment-X",
        "self-alignment-Y",
        ],
    "semi-tie-column-interface": [
        "head-direction",
        "positioning-done",
        "tie-configuration",
        ],
    "semi-tie-interface": [
        "control-points",
        "details",
        "direction",
        "head-direction",
        "thickness",
        ],
    "separation-item-interface": [
        "X-extent",
        "horizontal-skylines",
        "padding",
        "skyline-vertical-padding",
        ],
    "side-position-interface": [
        "add-stem-support",
        "direction",
        "horizon-padding",
        "minimum-space",
        "padding",
        "side-axis",
        "slur-padding",
        "staff-padding",
        "use-skylines",
        ],
    "slur-interface": [
        "annotation",
        "avoid-slur",
        "control-points",
        "dash-definition",
        "details",
        "direction",
        "eccentricity",
        "height-limit",
        "inspect-index",
        "inspect-quants",
        "line-thickness",
        "positions",
        "ratio",
        "thickness",
        ],
    "spaceable-grob-interface": [
        "allow-loose-spacing",
        "keep-inside-line",
        "measure-length",
        ],
    "spacing-interface": [],
    "spacing-options-interface": [
        "shortest-duration-space",
        "spacing-increment",
        ],
    "spacing-spanner-interface": [
        "average-spacing-wishes",
        "base-shortest-duration",
        "common-shortest-duration",
        "packed-spacing",
        "shortest-duration-space",
        "spacing-increment",
        "strict-grace-spacing",
        "strict-note-spacing",
        "uniform-stretching",
        ],
    "span-bar-interface": [
        "glyph-name",
        ],
    "spanner-interface": [
        "minimum-length",
        "minimum-length-after-break",
        "normalized-endpoints",
        "spanner-id",
        "to-barline",
        ],
    "staff-grouper-interface": [
        "staff-staff-spacing",
        "staffgroup-staff-spacing",
        ],
    "staff-spacing-interface": [
        "stem-spacing-correction",
        ],
    "staff-symbol-interface": [
        "break-align-symbols",
        "ledger-extra",
        "ledger-line-thickness",
        "ledger-positions",
        "line-count",
        "line-positions",
        "staff-space",
        "thickness",
        "width",
        ],
    "staff-symbol-referencer-interface": [
        "staff-position",
        ],
    "stanza-number-interface": [],
    "stem-interface": [
        "avoid-note-head",
        "beaming",
        "beamlet-default-length",
        "beamlet-max-length-proportion",
        "default-direction",
        "details",
        "direction",
        "double-stem-separation",
        "duration-log",
        "french-beaming",
        "length",
        "length-fraction",
        "max-beam-connect",
        "neutral-direction",
        "no-stem-extend",
        "positioning-done",
        "stem-begin-position",
        "stemlet-length",
        "thickness",
        ],
    "stem-tremolo-interface": [
        "beam-thickness",
        "beam-width",
        "direction",
        "flag-count",
        "length-fraction",
        "shape",
        "slope",
        ],
    "string-number-interface": [],
    "stroke-finger-interface": [
        "digit-names",
        ],
    "system-interface": [
        "labels",
        ],
    "system-start-delimiter-interface": [
        "collapse-height",
        "style",
        "thickness",
        ],
    "system-start-text-interface": [
        "long-text",
        "self-alignment-X",
        "self-alignment-Y",
        "text",
        ],
    "tab-note-head-interface": [
        "details",
        ],
    "text-interface": [
        "baseline-skip",
        "flag-style",
        "replacement-alist",
        "text",
        "text-direction",
        "word-space",
        ],
    "text-script-interface": [
        "avoid-slur",
        "script-priority",
        ],
    "tie-column-interface": [
        "positioning-done",
        "tie-configuration",
        ],
    "tie-interface": [
        "annotation",
        "avoid-slur",
        "control-points",
        "dash-definition",
        "details",
        "direction",
        "head-direction",
        "line-thickness",
        "neutral-direction",
        "staff-position",
        "thickness",
        ],
    "time-signature-interface": [
        "fraction",
        "style",
        ],
    "trill-pitch-accidental-interface": [],
    "trill-spanner-interface": [],
    "tuplet-bracket-interface": [
        "X-positions",
        "avoid-scripts",
        "bracket-flare",
        "bracket-visibility",
        "break-overshoot",
        "connect-to-neighbor",
        "direction",
        "edge-height",
        "edge-text",
        "full-length-padding",
        "full-length-to-extent",
        "gap",
        "padding",
        "positions",
        "shorten-pair",
        "staff-padding",
        "thickness",
        ],
    "tuplet-number-interface": [
        "avoid-slur",
        "direction",
        "knee-to-beam",
        ],
    "unbreakable-spanner-interface": [
        "breakable",
        ],
    "vaticana-ligature-interface": [
        "glyph-name",
        "thickness",
        ],
    "volta-bracket-interface": [
        "height",
        "shorten-pair",
        "thickness",
        ],
    "volta-interface": [],
    }
