import abjad


class {class_name}(abjad.AbjadObject):
    """
    """

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __call__(self):
        pass

    ### PUBLIC PROPERTIES ###
