"""
Abjad's top-level functions.
"""

from .activate import activate
from .annotate import annotate
from .attach import attach
from .deactivate import deactivate
from .detach import detach
from .f import f
from .graph import graph
from .inspect import inspect
from .iterate import iterate
from .label import label
from .mutate import mutate
from .new import new
from .override import override
from .parse import parse
from .persist import persist
from .play import play
from .select import select
from .sequence import sequence
from .setting import setting
from .show import show
from .tweak import tweak
